<?php

/**
 * Factory for component iterators.
 */
class EntityTemplateComponentIteratorFactory {

  /**
   * Get the plugin definitions.
   *
   * @return array
   *   The definitions keyed by name.
   */
  public static function getDefinitions() {
    $plugins = &drupal_static(__CLASS__.__METHOD__, array());
    if (!empty($plugins)) {
      return $plugins;
    }

    if ($cached = cache_get(__CLASS__.__METHOD__)) {
      $plugins = $cached->data;
    }
    else {
      $plugins = module_invoke_all('entity_template_component_iterator_plugin_info');
      drupal_alter('entity_template_component_iterator_plugin_info', $plugins);

      cache_set(__METHOD__, $plugins);
    }

    return $plugins;
  }

  /**
   * Get a plugin definition.
   *
   * @param string $plugin_id
   *   The plugin id.
   *
   * @return array
   *   A plugin definition.
   */
  public static function getDefinition($plugin_id) {
    return static::getDefinitions()[$plugin_id] ?? NULL;
  }

  /**
   * Create an instance of the plugin.
   *
   * @param string $plugin_id
   *   The plugin id.
   * @param array $configuration
   *   The configuration.
   *
   * @return \EntityTemplateComponentIteratorInterface
   */
  public static function createInstance($plugin_id, array $configuration = []) {
    $definition = static::getDefinition($plugin_id);

    $instance = new $definition['class']($plugin_id, $definition);
    if ($instance instanceof ConfigurableEntityTemplateComponentIteratorInterface) {
      $instance->setConfiguration($configuration);
    }

    return $instance;
  }

}
