<?php

interface EntityTemplateComponentIteratorInterface {

  /**
   * Gets the plugin_id of the plugin instance.
   *
   * @return string
   *   The plugin_id of the plugin instance.
   */
  public function getPluginId();

  /**
   * Gets the definition of the plugin implementation.
   *
   * @return array
   *   The plugin definition, as returned by the discovery object used by the
   *   plugin manager.
   */
  public function getPluginDefinition();

  /**
   * Get the additional parameters info that will be added to each iteration.
   *
   * @param array $parametersInfo
   *
   * @return array
   */
  public function getAdditionalParametersInfo(array $parametersInfo = []) : array;

  /**
   * Get an iterator returning parameter sets to be used by the component.
   *
   * @param array $parameters
   *   The parameters.
   *
   * @return \Iterator
   */
  public function getIterator(array $parameters) : \Iterator;

  /**
   * Set the component the iterator is acting on.
   *
   * @param \EntityTemplateComponentInterface $component
   *   The component.
   *
   * @return $this
   */
  public function setComponent(EntityTemplateComponentInterface $component) : EntityTemplateComponentIteratorInterface;

}

interface ConfigurableEntityTemplateComponentIteratorInterface extends EntityTemplateComponentIteratorInterface {

  /**
   * Get the configuration.
   *
   * @return array
   *   The configuration.
   */
  public function getConfiguration() : array;

  /**
   * Set the configuration.
   *
   * @param array $configuration
   *   The configuration.
   *
   * @return $this
   */
  public function setConfiguration(array $configuration) : ConfigurableEntityTemplateComponentIteratorInterface;

  /**
   * Build a configuration form.
   *
   * @param array $form
   * @param array $form_state
   *
   * @return array
   */
  public function configurationForm($form, &$form_state) : array;

  /**
   * Validate the configuration form.
   *
   * @param array $form
   * @param array $form_state
   *
   * @return void
   */
  public function configurationFormValidate($form, &$form_state);

  /**
   * Submit the configuration form.
   *
   * @param array $form
   * @param array $form_state
   *
   * @return void
   */
  public function configurationFormSubmit($form, &$form_state);

}

/**
 * Interface for a modifier that needs to know about parameters.
 */
interface ParameterAwareEntityTemplateComponentIteratorInterface extends EntityTemplateComponentIteratorInterface {

  /**
   * Set the parameters.
   *
   * @param array $parameters
   *   The parameters.
   *
   * @return $this
   */
  public function setParameters(array $parameters) : ParameterAwareEntityTemplateComponentIteratorInterface;

}
