<?php

class ListPropertyComponentIterator extends EntityTemplateComponentIteratorBase implements ConfigurableEntityTemplateComponentIteratorInterface {
  use ConfigurableEntityTemplateComponentIteratorTrait;

  /**
   * {@inheritDoc}
   */
  public function getAdditionalParametersInfo(array $parametersInfo = []): array {
    $wrapped_params = [];
    foreach ($parametersInfo as $key => $info) {
      $wrapped_params[$key] = entity_metadata_wrapper($info['type'], NULL, $info);
    }
    try {
      $list_wrapper = $this->getNestedWrapper($wrapped_params, $this->configuration['selector']);
    }
    catch (Exception $e) {
      $list_wrapper = entity_metadata_wrapper('list<struct>');
    }

    return [
      $this->configuration['item_name'] => [
        'type' => entity_property_list_extract_type($list_wrapper->type()) ?? $list_wrapper->type(),
        'label' => ucwords(str_replace('_', ' ', $this->configuration['item_name'])),
      ] + $list_wrapper->info(),
    ];
  }

  /**
   * {@inheritDoc}
   */
  public function getIterator(array $parameters): \Iterator {
    $selectors = explode(',', $this->configuration['selector']);

    $parameter_sets = [];
    foreach ($selectors as $selector) {
      $wrapped = $this->getNestedWrapper($parameters, $selector);
      if ($wrapped instanceof EntityListWrapper) {
        foreach ($wrapped as $item) {
          $parameter_sets[] = [
            $this->configuration['item_name'] => $item,
          ] + $parameters;
        }
      }
      else {
        $parameter_sets[] = [
          $this->configuration['item_name'] => $wrapped,
        ] + $parameters;
      }
    }

    return new \ArrayIterator($parameter_sets);
  }

  /**
   * {@inheritDoc}
   */
  public function configurationForm($form, &$form_state): array {
    $form['selector'] = [
      '#type' => 'textfield',
      '#title' => t('List to Iterate Over'),
      '#default_value' => $this->configuration['selector'],
      '#size' => 75,
    ];

    $form['item_name'] = [
      '#type' => 'textfield',
      '#title' => t('Item Name'),
      '#default_value' => $this->configuration['item_name'] ?: 'item',
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritDoc}
   */
  public function configurationFormSubmit($form, &$form_state) {
    $values = drupal_array_get_nested_value($form_state['values'], $form['#parents']);
    $this->configuration['selector'] = $values['selector'];
    $this->configuration['item_name'] = $values['item_name'] ?: 'item';
  }

  /**
   * Get a nested property.
   */
  protected function getNestedWrapper($wrapped_params, $selector) {
    $parts = explode(':', $selector);
    $namespace = str_replace('-', '_', array_shift($parts));
    $selected = $wrapped_params[$namespace];

    if (empty($selected)) {
      return FALSE;
    }

    try {
      while (($sub = array_shift($parts)) !== NULL) {
        try {
          $sub = str_replace('-', '_', $sub);
          $selected = $selected->get($sub);
        }
        catch (EntityMetadataWrapperException $e) {
          return FALSE;
        }
      }
    }
    catch (Exception $e) {
      return FALSE;
    }

    return $selected;
  }

}
